/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import nl.cloudfarming.client.fleet.model.Fleet;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.spi.project.ActionProvider;
import org.netbeans.spi.project.ProjectState;
import org.netbeans.spi.project.ui.support.CommonProjectActions;
import org.openide.filesystems.FileObject;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Wytse Visser & Maciek Dulkiewicz
 */
public class FleetProject implements Project {

    private final FileObject projectDir;
    private final ProjectState state;
    private Lookup lkp;
    private Fleet fleet;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.project");

    public FleetProject(FileObject dir, ProjectState state, Fleet fleet) {
        projectDir = dir;
        this.state = state;
        this.fleet = fleet;
    }

    @Override
    public FileObject getProjectDirectory() {
        return projectDir;
    }

    @Override
    public Lookup getLookup() {
        if (lkp == null) {
            lkp = Lookups.fixed(new Object[]{
                        state,
                        new ActionProviderImpl(),
                        new FleetDeleteAction(),
                        new FleetProject.FleetProjectInfo(),
                        new FleetProjectLogicalView(this),
                        new MachineServiceImpl()});
        }
        return lkp;
    }

    public Fleet getFleet() {
        return fleet;
    }

    private final class FleetProjectInfo implements ProjectInformation {

        @Override
        public Icon getIcon() {
            return new ImageIcon(ImageUtilities.loadImage(
                    FleetProjectFactory.ICON));
        }

        @Override
        public String getName() {
            return getProjectDirectory().getName();
        }

        @Override
        public String getDisplayName() {
            return getName();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public Project getProject() {
            return FleetProject.this;
        }
    }

    private final class ActionProviderImpl implements ActionProvider {

        @Override
        public String[] getSupportedActions() {
            return new String[]{ActionProvider.COMMAND_DELETE};
        }

        @Override
        public void invokeAction(String string, Lookup lookup) throws IllegalArgumentException {
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_DELETE)) {
                getLookup().lookup(FleetDeleteAction.class).actionPerformed(null);
            }
        }

        @Override
        public boolean isActionEnabled(String command, Lookup lookup) throws IllegalArgumentException {
            switch (command) {
                case ActionProvider.COMMAND_DELETE:
                    return true;
                default:
                    throw new IllegalArgumentException(command);
            }
        }
    }

    @NbBundle.Messages({"warningMessage=Are you sure you want to delete the fleet %s?", "dialogTitle=Warning!", "delete_action_name=Delete"})
    public final class FleetDeleteAction extends AbstractAction {

        @Override
        public void actionPerformed(ActionEvent e) {
            final String warningMsg = String.format(Bundle.warningMessage(), getFleet().getName());
            int option = JOptionPane.showConfirmDialog(null, warningMsg, Bundle.dialogTitle(), JOptionPane.YES_NO_OPTION);
            if (option == JOptionPane.OK_OPTION) {
                CommonProjectActions.closeProjectAction().actionPerformed(null);
                try {
                    getProjectDirectory().delete();
                } catch (IOException ex) {
                    LOGGER.log(Level.WARNING, "Could not remove fleet.", ex); //NOI18N
                }
            }
        }
    }
}
